//===========================================================================
//	CMaxMidiIn Class Implementation
//
//	(C) Copyright, Paul A. Messick, 1996
//===========================================================================
#include "stdafx.h"
#include "MaxMidi.h"

BEGIN_MESSAGE_MAP(CMaxMidiIn, CWnd)
	//{{AFX_MSG_MAP(CMaxMidiIn)
	ON_MESSAGE(MIDI_DATA, OnMidiData)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------
//	CMaxMidiIn Constructors
//---------------------------------------------------------------------------
CMaxMidiIn::CMaxMidiIn()
{
	hDevice = 0;
	fIsOpen = FALSE;
	Description[0] = 0;
	hParentWnd = NULL;

#ifdef _SYNC
	pSync = 0;
#endif
#ifdef _TRACK
	pTrack = 0;
#endif
}

CMaxMidiIn::CMaxMidiIn(HWND hParentWnd, WORD wDeviceID)
{
	hDevice = 0;
	fIsOpen = FALSE;
	Description[0] = 0;
#ifdef _SYNC
	pSync = 0;
#endif
#ifdef _TRACK
	pTrack = 0;
#endif

	Attach(hParentWnd);
	Open(wDeviceID);
}

#ifdef _SYNC
CMaxMidiIn::CMaxMidiIn(HWND hParentWnd, WORD wDeviceID, CMaxMidiSync* pSync, DWORD dwFlags)
{
#ifdef _TRACK
	pTrack = 0;
#endif

	Attach(hParentWnd);
	Attach(pSync);
	Open(wDeviceID, dwFlags);
}
#endif

BOOL CMaxMidiIn::CreateWnd(void)
{
	RECT r;

	r.left = r.top = r.right = r.bottom = 0;

	return Create(NULL, "CMaxMidiInWnd", WS_CHILD, r, FromHandlePermanent(hParentWnd), 0);
}

//---------------------------------------------------------------------------
//	GetIDFromName
//---------------------------------------------------------------------------
WORD CMaxMidiIn::GetIDFromName(LPSTR lpszDesc)
{
	WORD id;
	char thisDesc[MAXPNAMELEN];
	WORD MaxDevs = GetNumInDevices();

	for(id = 0; id < MaxDevs; id++)
	{
		GetMidiInDescription(id, thisDesc);
		if(strcmp(thisDesc, lpszDesc) == 0)
			return id;

		id++;
	}

	return ERR_NOMATCH;
}

//---------------------------------------------------------------------------
//	Attach/Detach
//---------------------------------------------------------------------------
void CMaxMidiIn::Attach(HWND hParentWnd)
{
	CMaxMidiIn::hParentWnd = hParentWnd;
	CreateWnd();
}

#ifdef _SYNC
void CMaxMidiIn::Attach(CMaxMidiSync* pSync)
{
	CMaxMidiIn::pSync = pSync;
}

void CMaxMidiIn::Detach(CMaxMidiSync* pSync)
{
	// detach this sync device, if it is attached
	if(pSync == CMaxMidiIn::pSync)
		pSync = NULL;

	// close and reopen the device
	if(fIsOpen)
	{
		Close();
		Open(wDeviceID, dwFlags);
	}
}
#endif

#ifdef _TRACK
void CMaxMidiIn::Attach(CMaxMidiTrack* pTrack)
{
	// detach the track if one was already attached
	if(CMaxMidiIn::pTrack != NULL)
		CMaxMidiIn::pTrack->Detach(this);

	// attach the new track
	CMaxMidiIn::pTrack = pTrack;

	// tell the track about this input device
	pTrack->Attach(this);
}

void CMaxMidiIn::Detach(CMaxMidiTrack* pTrack)
{
	// detach this track, if it is attached
	if(pTrack == CMaxMidiIn::pTrack)
		pTrack = NULL;
}
#endif

//---------------------------------------------------------------------------
//	CMaxMidiIn Destructor
//---------------------------------------------------------------------------
CMaxMidiIn::~CMaxMidiIn()
{
	Close();
}

//---------------------------------------------------------------------------
//	Open
//---------------------------------------------------------------------------
BOOL CMaxMidiIn::Open(WORD wDeviceID, DWORD dwFlags)
{
	HSYNC hSync = 0;

	// get the description of the device
	GetMidiInDescription(wDeviceID, Description);

#ifdef _SYNC
	// get the sync device handle
	if(CMaxMidiIn::pSync)
		hSync = CMaxMidiIn::pSync->GetHSync();
#endif

	if(!fIsOpen)
	{
		CMaxMidiIn::dwFlags = dwFlags;
		CMaxMidiIn::wDeviceID = wDeviceID;
		hDevice = OpenMidiIn(m_hWnd, wDeviceID, hSync, dwFlags);
	}

	if(hDevice > MXMIDIERR_MAXERR)
		fIsOpen = TRUE;
	else
		hDevice = 0;

	return fIsOpen;
}

//---------------------------------------------------------------------------
//	Close
//---------------------------------------------------------------------------
void CMaxMidiIn::Close(void)
{
	if(fIsOpen)
	{
		Stop();
		CloseMidiIn(hDevice);
	}

	fIsOpen = FALSE;
}

//---------------------------------------------------------------------------
//	Information Functions
//---------------------------------------------------------------------------
BOOL CMaxMidiIn::IsOpen(void)
{
	return fIsOpen;
}

LPSTR CMaxMidiIn::GetDescription(void)
{
	return Description;
}

int CMaxMidiIn::GetNumDevices(void)
{
	return GetNumInDevices();
}

//---------------------------------------------------------------------------
//	Start/Stop
//---------------------------------------------------------------------------
void CMaxMidiIn::Start(void)
{
	if(fIsOpen)
	{
		fIsStarted = TRUE;
		StartMidiIn(hDevice);
	}
}

void CMaxMidiIn::Stop(void)
{
	if(fIsOpen)
	{
		fIsStarted = FALSE;
		StopMidiIn(hDevice);
	}
}

void CMaxMidiIn::Reset(void)
{
	if(fIsOpen && fIsStarted)
	{
		StopMidiIn(hDevice);
		StartMidiIn(hDevice);
	}
}

//---------------------------------------------------------------------------
//	Get Data
//---------------------------------------------------------------------------
LPMIDIEVENT CMaxMidiIn::Get(void)
{
	return GetMidiIn(hDevice);
}

/////////////////////////////////////////////////////////////////////////////
// CMaxMidiIn message handlers

//---------------------------------------------------------------------------
//	OnMimData
//---------------------------------------------------------------------------
LRESULT CMaxMidiIn::OnMidiData(WPARAM wParam, LPARAM lParam)
{
	LPMIDIEVENT lpEvent;

	// send the data to the track object, if attached
	while((lpEvent = Get()) != 0)
	{
		if(ProcessMidiData(lpEvent) == TRUE)
#ifdef _TRACK
			if(pTrack)
				pTrack->Write(lpEvent);
#else
			;
#endif
	}

	return 0;
}
