//===========================================================================
//	CMaxMidiSync Class Implementation
//
//	(C) Copyright, Paul A. Messick, 1996
//===========================================================================
#include "stdafx.h"
#include "MaxMidi.h"

BEGIN_MESSAGE_MAP(CMaxMidiSync, CWnd)
	//{{AFX_MSG_MAP(CMaxMidiSync)
	ON_MESSAGE(MIDI_BEAT, OnMidiBeat)
	ON_MESSAGE(SYNC_DONE, OnSyncDone)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------
//	CMaxMidiSync Constructors
//---------------------------------------------------------------------------
CMaxMidiSync::CMaxMidiSync()
{
	hDevice = 0;
	fIsOpen = FALSE;
	fRunning = FALSE;
	hParentWnd = NULL;
}

CMaxMidiSync::CMaxMidiSync(HWND hParentWnd)
{
	hDevice = 0;
	fIsOpen = FALSE;
	fRunning = FALSE;
	Attach(hParentWnd);
}

CMaxMidiSync::CMaxMidiSync(HWND hParentWnd, WORD mode, WORD timerPeriod)
{
	fRunning = FALSE;
	Attach(hParentWnd);
	Open(mode, timerPeriod);
}

BOOL CMaxMidiSync::CreateWnd(void)
{
	RECT r;

	r.left = r.top = r.right = r.bottom = 0;

	return Create(NULL, "CMaxMidiSyncWnd", WS_CHILD, r, FromHandlePermanent(hParentWnd), 0);
}

void CMaxMidiSync::Attach(HWND hParentWnd)
{
	CMaxMidiSync::hParentWnd = hParentWnd;
	CreateWnd();
}

//---------------------------------------------------------------------------
//	CMaxMidiSync Destructors
//---------------------------------------------------------------------------
CMaxMidiSync::~CMaxMidiSync()
{
	Stop();
	Close();
}

//---------------------------------------------------------------------------
//	Open/Close
//---------------------------------------------------------------------------
BOOL CMaxMidiSync::Open(WORD mode, WORD timerPeriod)
{
	if(!fIsOpen)
		hDevice = OpenSync(0, m_hWnd, mode, timerPeriod);

	if(hDevice > MXMIDIERR_MAXERR)
	{
		fIsOpen = TRUE;
		CurrentMode = mode;
		CurrentPeriod = timerPeriod;
	}
	else
		hDevice = 0;

	return fIsOpen;
}

void CMaxMidiSync::Close(void)
{
	if(fIsOpen)
		CloseSync(hDevice);

	fRunning = FALSE;
	fIsOpen = FALSE;
}

//---------------------------------------------------------------------------
//	Mode/Period
//---------------------------------------------------------------------------
BOOL CMaxMidiSync::Mode(WORD mode)
{
	if(fIsOpen)
	{
		hDevice = OpenSync(hDevice, m_hWnd, mode, USE_CURRENT);

		if(hDevice > MXMIDIERR_MAXERR)
		{
			fIsOpen = TRUE;
			CurrentMode = mode;
		}
		else
		{
			hDevice = 0;
			fIsOpen = FALSE;
		}
	}

	return fIsOpen;
}

BOOL CMaxMidiSync::Period(WORD period)
{
	if(fIsOpen)
	{
		hDevice = OpenSync(hDevice, m_hWnd, USE_CURRENT, period);

		if(hDevice > MXMIDIERR_MAXERR)
		{
			fIsOpen = TRUE;
			CurrentPeriod = period;
		}
		else
		{
			hDevice = 0;
			fIsOpen = FALSE;
		}
	}

	return fIsOpen;
}

//---------------------------------------------------------------------------
//	Information Functions
//---------------------------------------------------------------------------
BOOL CMaxMidiSync::IsOpen(void)
{
	return fIsOpen;
}

//---------------------------------------------------------------------------
//	Start/ReStart
//---------------------------------------------------------------------------
void CMaxMidiSync::Start(void)
{
	if(fIsOpen)
	{
		StartSync(hDevice);
		fRunning = TRUE;
	}
}

void CMaxMidiSync::ReStart(void)
{
	if(fIsOpen)
	{
		ReStartSync(hDevice);
		fRunning = TRUE;
	}
}

//---------------------------------------------------------------------------
//	Stop/Pause
//---------------------------------------------------------------------------
void CMaxMidiSync::Stop(void)
{
	if(fIsOpen)
	{
		StopSync(hDevice);
		fRunning = FALSE;
	}
}

void CMaxMidiSync::Pause(BOOL reset)
{
	if(fIsOpen)
	{
		PauseSync(hDevice, reset);
		fRunning = FALSE;
	}
}

//---------------------------------------------------------------------------
//	Tempo
//---------------------------------------------------------------------------
DWORD CMaxMidiSync::Tempo(void)
{
	return GetTempo(hDevice);
}

BOOL CMaxMidiSync::Tempo(DWORD tempo)
{
	return (SetTempo(hDevice, tempo) == 0);
}

//---------------------------------------------------------------------------
//	Resolution
//---------------------------------------------------------------------------
WORD CMaxMidiSync::Resolution(void)
{
	return GetResolution(hDevice);
}

void CMaxMidiSync::Resolution(WORD res)
{
	SetResolution(hDevice, res);
}

//---------------------------------------------------------------------------
//	Convert Tempo
//---------------------------------------------------------------------------
DWORD CMaxMidiSync::Convert(double tempo)
{
	// can't allow zero tempo!
	if(tempo == 0.0)
		tempo = 120.0;

	return (DWORD)(60000000.0/tempo);
}

double CMaxMidiSync::Convert(DWORD tempo)
{
	return (60000000.0/(double)tempo);
}

/////////////////////////////////////////////////////////////////////////////
// CMaxMidiSync message handlers

//---------------------------------------------------------------------------
//	OnMidiBeat
//---------------------------------------------------------------------------
LRESULT CMaxMidiSync::OnMidiBeat(WPARAM wParam, LPARAM lParam)
{
	ProcessMidiBeat();
	return 0;
}

//---------------------------------------------------------------------------
//	OnSyncDone
//---------------------------------------------------------------------------
LRESULT CMaxMidiSync::OnSyncDone(WPARAM wParam, LPARAM lParam)
{
	ProcessSyncDone();
	return 0;
}
