//===========================================================================
//	CMidiOutDeviceMenu Class Implementation
//
//	(C) Copyright, Paul A. Messick, 1996
//===========================================================================
#include "stdafx.h"
#include "MaxMidi.h"

//---------------------------------------------------------------------------
//	CMidiOutDeviceMenu Constructor
//---------------------------------------------------------------------------
CMidiOutDeviceMenu::CMidiOutDeviceMenu(HMENU hWndMenu, UINT position, LPSTR mname, UINT baseMsg)
{
	Create(hWndMenu, position, mname, baseMsg);
}

//---------------------------------------------------------------------------
//	Create
//---------------------------------------------------------------------------
void CMidiOutDeviceMenu::Create(HMENU hWndMenu, UINT position, LPSTR mname, UINT baseMsg)
{
	char name[MAXPNAMELEN];
	WORD wDevID;

	// create a new popup menu
	hPopupMenu = CreateMenu();
	idm_base = baseMsg;

	// and insert in the existing window menu
	InsertMenu(hWndMenu, position, MF_BYPOSITION | MF_POPUP | MF_ENABLED, (UINT)hPopupMenu, mname);

	// add all of the available input devices to the menu
	// nMaxItems will hold the total number of devices found
	nMaxDevices = GetNumOutDevices();
	for(wDevID = 0; wDevID < nMaxDevices; wDevID++)
	{
		GetDeviceName(wDevID, name);
		AppendMenu(hPopupMenu, MF_ENABLED | MF_STRING, baseMsg + wDevID, name);
	}

	// try to add the MIDI Mapper, if present
	MapperID = 0;
	if(GetDeviceName((WORD)MIDI_MAPPER, name) == TRUE)
	{
		AppendMenu(hPopupMenu, MF_ENABLED | MF_STRING, baseMsg + nMaxDevices, name);
		MapperID = baseMsg + nMaxDevices;
		nMaxDevices++;
	}
}

//---------------------------------------------------------------------------
//	GetDeviceName
//---------------------------------------------------------------------------
BOOL CMidiOutDeviceMenu::GetDeviceName(WORD wDeviceID, LPSTR name)
{
	return GetMidiOutDescription(wDeviceID, name);
}

//---------------------------------------------------------------------------
//	SelectDevice
//---------------------------------------------------------------------------
BOOL CMidiOutDeviceMenu::SelectDevice(UINT id) 
{
	int i;
	CMenu m;
	BOOL ItsOpen;

	// ignore this if no device is attached or if the id is not in range
	if(MidiOut == NULL || ((int)id > nMaxDevices && ((int)id < idm_base) || ((int)id > idm_base + nMaxDevices)))
		return FALSE;

	// get a temp CMenu to modify the menu state
	CMenu* Menu = m.FromHandle(hPopupMenu);

	// if the id is between 0 and the number of devices
	// then select the input based on the index, otherwise
	// it is a menu selection
	if((int)id <= nMaxDevices)
		id += idm_base;

	for(i = idm_base; i < (int)(idm_base + nMaxDevices); i++)
		if(i == (int)id)
		{
			//-----------------------------------------------------
			// close the currently open output device
			//-----------------------------------------------------
			MidiOut->Reset();
			MidiOut->Close();

			//-----------------------------------------------------
			// open midi out for use
			//-----------------------------------------------------
			if(id == MapperID)
				ItsOpen = MidiOut->Open((WORD)MIDI_MAPPER);
			else
				ItsOpen = MidiOut->Open(id - idm_base);

			Menu->CheckMenuItem(i, (ItsOpen ? MF_CHECKED : MF_UNCHECKED));
		}
		else
			Menu->CheckMenuItem(i, MF_UNCHECKED);
	
	return TRUE;
}

