//===========================================================================
//	CMaxMidiOut Class Implementation
//
//	(C) Copyright, Paul A. Messick, 1996
//===========================================================================
#include "stdafx.h"
#include "MaxMidi.h"

BEGIN_MESSAGE_MAP(CMaxMidiOut, CWnd)
	//{{AFX_MSG_MAP(CMaxMidiOut)
	ON_MESSAGE(OUTBUFFER_READY, OnOutBufferReady)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------
//	CMaxMidiOut Constructors
//---------------------------------------------------------------------------
CMaxMidiOut::CMaxMidiOut()
{
	hDevice = 0;
	fIsOpen = FALSE;
	Description[0] = 0;
	hParentWnd = NULL;
#ifdef _SYNC
	pSync = NULL;
#endif
#ifdef _TRACK
	pTrackList = NULL;
	nTracks = 0;
	numEvents = 0;
	lpMerge = NULL;
#endif
}

CMaxMidiOut::CMaxMidiOut(HWND hParentWnd, WORD wDeviceID)
{
	hDevice = 0;
	fIsOpen = FALSE;
	Description[0] = 0;
#ifdef _SYNC
	pSync = NULL;
#endif
#ifdef _TRACK
	pTrackList = NULL;
	nTracks = 0;
	numEvents = 0;
	lpMerge = NULL;
#endif

	Attach(hParentWnd);
	Open(wDeviceID);
}

#ifdef _SYNC
CMaxMidiOut::CMaxMidiOut(HWND hParentWnd, WORD wDeviceID, CMaxMidiSync* pSync, DWORD dwFlags)
{
#ifdef _TRACK
	pTrackList = NULL;
	nTracks = 0;
	numEvents = 0;
	lpMerge = NULL;
#endif
	Attach(hParentWnd);
	Attach(pSync);
	Open(wDeviceID, dwFlags);
}
#endif

BOOL CMaxMidiOut::CreateWnd(void)
{
	RECT r;

	r.left = r.top = r.right = r.bottom = 0;

	return Create(NULL, "CMaxMidiOutWnd", WS_CHILD, r, FromHandlePermanent(hParentWnd), 0);
}

void CMaxMidiOut::Attach(HWND hParentWnd)
{
	CMaxMidiOut::hParentWnd = hParentWnd;
	CreateWnd();
}

//---------------------------------------------------------------------------
//	GetIDFromName
//---------------------------------------------------------------------------
WORD CMaxMidiOut::GetIDFromName(LPSTR lpszDesc)
{
	WORD id;
	char thisDesc[MAXPNAMELEN];
	WORD MaxDevs = GetNumOutDevices();

	for(id = 0; id < MaxDevs; id++)
	{
		GetMidiOutDescription(id, thisDesc);
		if(strcmp(thisDesc, lpszDesc) == 0)
			return id;

		id++;
	}

	return ERR_NOMATCH;
}

#ifdef _SYNC
//---------------------------------------------------------------------------
//	Attach/Detach sync
//---------------------------------------------------------------------------
void CMaxMidiOut::Attach(CMaxMidiSync* pSync)
{
	CMaxMidiOut::pSync = pSync;
}

void CMaxMidiOut::Detach(CMaxMidiSync* pSync)
{
	if(pSync == CMaxMidiOut::pSync)
		CMaxMidiOut::pSync = NULL;

	// close and reopen the device
	if(fIsOpen)
	{
		Close();
		Open(wDeviceID, dwFlags);
	}
}
#endif

#ifdef _TRACK
//---------------------------------------------------------------------------
//	Attach/Detach Tracks
//---------------------------------------------------------------------------
void CMaxMidiOut::Attach(CMaxMidiTrack* pTrack)
{
	// expand the track list for one more entry
	nTracks++;
	if(pTrackList)
		// add an entry if list exists
		pTrackList = (TrackMerge*)GlobalReAlloc(pTrackList, nTracks * sizeof(TrackMerge), GPTR | GMEM_MOVEABLE);
	else
		// create the list
		pTrackList = (TrackMerge*)GlobalAlloc(GPTR, sizeof(TrackMerge));

	// append this track to the end of the track list
	pTrackList[nTracks - 1].pTrack = pTrack;
	pTrackList[nTracks - 1].lastEvent = 0;
	pTrackList[nTracks - 1].fInSysex = FALSE;
	pTrackList[nTracks - 1].bufSize = MERGE_BUFFER_SIZE;

	// tell the track about this device
	pTrack->Attach(this);
}

BOOL CMaxMidiOut::Detach(CMaxMidiTrack* pTrack)
{
	int i, j;

	// make sure there is at least one track to delete
	if(nTracks == 0)
		return FALSE;

	// search for the specified track in the track list
	for(i = 0; i < nTracks; i++)
	{
		if(pTrackList[i].pTrack == pTrack)
		{
			// move all of the following tracks down by one entry
			// thus removing the offending track
			nTracks--;
			for(j = i; j < nTracks; j++)
				pTrackList[j] = pTrackList[j + 1];

			// shrink the list
			if(nTracks == 0)
			{
				GlobalFree(pTrackList);
				pTrackList = NULL;
			}
			else
				pTrackList = (TrackMerge*)GlobalReAlloc(pTrackList, nTracks * sizeof(TrackMerge), GPTR | GMEM_MOVEABLE);

			// it's removed!
			return TRUE;
		}
	}

	// else, this track is not in the list!
	return FALSE;
}
#endif

//---------------------------------------------------------------------------
//	CMaxMidiOut Destructor
//---------------------------------------------------------------------------
CMaxMidiOut::~CMaxMidiOut()
{
	Close();

#ifdef _TRACK
	// detach any attached tracks
	// *** V1.59 DLLs ***
	// detach will remove the track from the list and move the other tracks
	// up in the list. To detach all of the tracks, remove the first one over
	// and over until all have been removed.
	int nT = nTracks;
	for(int i = 0; i < nT; i++)
		Detach(pTrackList[0].pTrack);

	// free the merge buffer, if it exists
	if(lpMerge)
		GlobalFree(lpMerge);
#endif
}

//---------------------------------------------------------------------------
//	Open/Close
//---------------------------------------------------------------------------
BOOL CMaxMidiOut::Open(WORD wDeviceID, DWORD dwFlags)
{
	HSYNC hSync = 0;

	// get the description of the device
	GetMidiOutDescription(wDeviceID, Description);

#ifdef _SYNC
	// get the sync device handle
	if(pSync)
		hSync = pSync->GetHSync();
#endif

	if(!fIsOpen)
	{
		CMaxMidiOut::dwFlags = dwFlags;
		CMaxMidiOut::wDeviceID = wDeviceID;
		hDevice = OpenMidiOut(m_hWnd, wDeviceID, hSync, dwFlags);
	}

	if(hDevice > MXMIDIERR_MAXERR)
		fIsOpen = TRUE;
	else
		hDevice = 0;

	return fIsOpen;
}

void CMaxMidiOut::Close(void)
{
	if(fIsOpen)
	{
		Reset();
		CloseMidiOut(hDevice);
	}

	fIsOpen = FALSE;
}

//---------------------------------------------------------------------------
//	Information Functions
//---------------------------------------------------------------------------
BOOL CMaxMidiOut::IsOpen(void)
{
	return fIsOpen;
}

LPSTR CMaxMidiOut::GetDescription(void)
{
	return Description;
}

int CMaxMidiOut::GetNumDevices(void)
{
	return GetNumOutDevices();
}

//---------------------------------------------------------------------------
//	Put
//---------------------------------------------------------------------------
BOOL CMaxMidiOut::Put(LPMIDIEVENT lpEvent)
{
	if(fIsOpen)
		return (PutMidiOut(hDevice, lpEvent) == 0);

	return TRUE;		// act as though it accepts everything
						// in case Put is called when not open
}

//---------------------------------------------------------------------------
//	Reset/Flush
//---------------------------------------------------------------------------
void CMaxMidiOut::Reset(void)
{
	if(fIsOpen)
		ResetMidiOut(hDevice);
}

void CMaxMidiOut::Flush(void)
{
	if(fIsOpen)
		FlushMidiOut(hDevice);
}

#ifdef _TRACK
//---------------------------------------------------------------------------
//	Track Merge
//---------------------------------------------------------------------------
LPMIDIEVENT CMaxMidiOut::MergeTracks(void)
{
	int track;
	DWORD time;
	int minTrack;
	int outEvents;
	BOOL tracksDone = FALSE;
	int i;
	BOOL fAnyInSysex = FALSE;
	int SysexTrack;
	DWORD NewLastAbs;

	// create the merge buffer that will receive the merged events
	if(lpMerge == NULL)
		lpMerge = (LPMIDIEVENT)GlobalAlloc(GPTR, MERGE_BUFFER_SIZE * sizeof(MidiEvent));

	// get a block of events from each of the tracks, in absolute time
	for(track = 0; track < nTracks; track++)
	{
		// always try to read something, even if the buffer was empty last time
		pTrackList[track].bufSize = MERGE_BUFFER_SIZE;
		pTrackList[track].pAbsBuf = pTrackList[track].pTrack->GetAbsBuffer(pTrackList[track].lastEvent, &pTrackList[track].bufSize);

		// initialize the track structs for the search
		pTrackList[track].thisEvent = 0;

		// if this track is currently playing sysex, it blocks all other tracks
		if(pTrackList[track].pAbsBuf && pTrackList[track].fInSysex)
		{
			fAnyInSysex = TRUE;
			SysexTrack = track;
		}
	}

	// search through the tracks, merging events in ascending order, based on
	// abs time, into the merge buffer
	outEvents = 0;

	while(outEvents < MERGE_BUFFER_SIZE && !tracksDone)
	{
		// assume the tracks are empty to start
		tracksDone = TRUE;

		// if one of the tracks is in the middle of a sysex, stick on that track
		if(fAnyInSysex)
		{
			minTrack = SysexTrack;

			if(pTrackList[minTrack].pAbsBuf != NULL && pTrackList[minTrack].thisEvent < pTrackList[minTrack].bufSize)
			{
				time = pTrackList[minTrack].pAbsBuf[pTrackList[minTrack].thisEvent].time;
				tracksDone = FALSE;
			}
		}
		else
		// get the time of the first event in the first track that has an event
		for(minTrack = 0; minTrack < nTracks; minTrack++)
		{
			if(pTrackList[minTrack].pAbsBuf != NULL && pTrackList[minTrack].thisEvent < pTrackList[minTrack].bufSize)
			{
				time = pTrackList[minTrack].pAbsBuf[pTrackList[minTrack].thisEvent].time;
				tracksDone = FALSE;
 				break;
			}
		}

		if(!tracksDone)
		{
			// start the search from the next track, but
			// if one of the tracks is in the middle of a sysex,
			// remain on that track
			if(!fAnyInSysex)
				for(track = minTrack + 1; track < nTracks; track++)
				{
					// skip tracks that are empty
					if(pTrackList[track].pAbsBuf != NULL && pTrackList[track].thisEvent != pTrackList[track].bufSize)
					{
						// if the time of the next event in this track is less than
						// the current time, this is the new event
						if(pTrackList[track].pAbsBuf[pTrackList[track].thisEvent].time < time)
						{
							minTrack = track;
							time = pTrackList[track].pAbsBuf[pTrackList[track].thisEvent].time;
						}
					}
				}

			// found the next event, copy it over
			lpMerge[outEvents] = pTrackList[minTrack].pAbsBuf[pTrackList[minTrack].thisEvent];
			pTrackList[minTrack].thisEvent++;

			// is this the start or end of a sysex in this track?
			switch(lpMerge[outEvents].data1)
			{
				case SYSEX:
					pTrackList[minTrack].fInSysex = TRUE;
					fAnyInSysex = TRUE;
					SysexTrack = minTrack;
					break;

				case EOX:
					pTrackList[minTrack].fInSysex = FALSE;
					fAnyInSysex = FALSE;
					break;
			}

			outEvents++;
		}
	}

	// update all of the lastEvent indices and free the abs buffers
	for(track = 0; track < nTracks; track++)
	{
		pTrackList[track].lastEvent += pTrackList[track].thisEvent;
		if(pTrackList[track].pAbsBuf)
			GlobalFree(pTrackList[track].pAbsBuf);
	}

	// convert the merge buffer back to delta times
	if(outEvents)
	{
		NewLastAbs = lpMerge[outEvents - 1].time;

		for(i = outEvents - 1; i >= 1; i--)
		{
			lpMerge[i].time -= lpMerge[i - 1].time;

			// if the one of the tracks was playing a large sysex, the timestamp
			// for the next event in the stream might end up with a negative
			// timestamp, since its time would logically be during the sysex.
			// We have to catch up the tracks here... it stinks, I know, but it
			// is the best we can do under the circumstances (sysex, that is).
			if((int)lpMerge[i].time < 0)
				lpMerge[i].time = 0;
		}

		lpMerge[0].time -= LastAbs;
		LastAbs = NewLastAbs;
	}

	numEvents = outEvents;
	return lpMerge;
}

void CMaxMidiOut::MergeOut(void)
{
	if(nTracks)
	{
		// any events still in merge buffer?
		if(numEvents == outPtr)
		{
			// no, need to merge some more tracks
			// numEvents is set in MergeTracks()
			MergeTracks();
			outPtr = 0;
		}

		// pump out as much data as the MidiOut will accept
		while(outPtr < numEvents && Put(&lpMerge[outPtr]))
		{
			outPtr++;

			// merge more if needed
			if(numEvents == outPtr)
			{
				MergeTracks();
				outPtr = 0;
			}
		}

	}
}

void CMaxMidiOut::StartOut(void)
{
	int track;

	// rewind the tracks and start again
	for(track = 0; track < nTracks; track++)
	{
		pTrackList[track].lastEvent = 0;
		pTrackList[track].fInSysex = FALSE;
		pTrackList[track].bufSize = MERGE_BUFFER_SIZE;
	}

	LastAbs = 0;
	numEvents = outPtr = 0;
	MergeOut();
}
#endif

/////////////////////////////////////////////////////////////////////////////
// CMaxMidiOut message handlers

//---------------------------------------------------------------------------
//	OnOutBufferReady
//---------------------------------------------------------------------------
LRESULT CMaxMidiOut::OnOutBufferReady(WPARAM wParam, LPARAM hMidiOut)
{
#ifdef _TRACK
	// output events from any attached tracks
	MergeOut();
#endif

	ProcessOutBufferReady();
	return 0;
}

